@@ -1,9 +1,9 @@
 /*
- *	$Id: lspci.c,v 1.36 2000/04/21 11:58:00 mj Exp $
+ *	$Id: lspci.c,v 1.41 2002/03/30 15:39:24 mj Exp $
  *
  *	Linux PCI Utilities -- List All PCI Devices
  *
- *	Copyright (c) 1997--1999 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
+ *	Copyright (c) 1997--2002 Martin Mares <mj@ucw.cz>
  *
  *	Can be freely distributed and used under the terms of the GNU GPL.
  */
@@ -375,26 +375,36 @@ show_bases(struct device *d, int cnt)
 static void
 show_pm(struct device *d, int where, int cap)
 {
-  int t;
+  int t, b;
+  static int pm_aux_current[8] = { 0, 55, 100, 160, 220, 270, 320, 375 };
 
   printf("Power Management version %d\n", cap & PCI_PM_CAP_VER_MASK);
   if (verbose < 2)
     return;
-  printf("\t\tFlags: PMEClk%c AuxPwr%c DSI%c D1%c D2%c PME%c\n",
+  printf("\t\tFlags: PMEClk%c DSI%c D1%c D2%c AuxCurrent=%dmA PME(D0%c,D1%c,D2%c,D3hot%c,D3cold%c)\n",
 	 FLAG(cap, PCI_PM_CAP_PME_CLOCK),
-	 FLAG(cap, PCI_PM_CAP_AUX_POWER),
 	 FLAG(cap, PCI_PM_CAP_DSI),
 	 FLAG(cap, PCI_PM_CAP_D1),
 	 FLAG(cap, PCI_PM_CAP_D2),
-	 FLAG(cap, PCI_PM_CAP_PME));
+	 pm_aux_current[(cap >> 6) & 7],
+	 FLAG(cap, PCI_PM_CAP_PME_D0),
+	 FLAG(cap, PCI_PM_CAP_PME_D1),
+	 FLAG(cap, PCI_PM_CAP_PME_D2),
+	 FLAG(cap, PCI_PM_CAP_PME_D3_HOT),
+	 FLAG(cap, PCI_PM_CAP_PME_D3_COLD));
   config_fetch(d, where + PCI_PM_CTRL, PCI_PM_SIZEOF - PCI_PM_CTRL);
   t = get_conf_word(d, where + PCI_PM_CTRL);
-  printf("\t\tStatus: D%d PME-Enable%c DSel=%x DScale=%x PME%c\n",
+  printf("\t\tStatus: D%d PME-Enable%c DSel=%d DScale=%d PME%c\n",
 	 t & PCI_PM_CTRL_STATE_MASK,
 	 FLAG(t, PCI_PM_CTRL_PME_ENABLE),
 	 (t & PCI_PM_CTRL_DATA_SEL_MASK) >> 9,
 	 (t & PCI_PM_CTRL_DATA_SCALE_MASK) >> 13,
 	 FLAG(t, PCI_PM_CTRL_PME_STATUS));
+  b = get_conf_byte(d, where + PCI_PM_PPB_EXTENSIONS);
+  if (b)
+    printf("\t\tBridge: PM%c B3%c\n",
+	   FLAG(t, PCI_PM_BPCC_ENABLE),
+	   FLAG(~t, PCI_PM_PPB_B2_B3));
 }
 
 static void
@@ -403,7 +413,7 @@ format_agp_rate(int rate, char *buf)
   char *c = buf;
   int i;
 
-  for(i=0; i<2; i++)
+  for(i=0; i<=2; i++)
     if (rate & (1 << i))
       {
 	if (c != buf)
@@ -447,6 +457,86 @@ show_agp(struct device *d, int where, int cap)
 	 rate);
 }
 
+static void
+show_pcix_nobridge(struct device *d, int where)
+{
+  u16 command = get_conf_word(d, where + PCI_PCIX_COMMAND);
+  u32 status = get_conf_long(d, where + PCI_PCIX_STATUS);
+  printf("PCI-X non-bridge device.\n");
+  if (verbose < 2)
+    return;
+  printf("\t\tCommand: DPERE%c ERO%c RBC=%d OST=%d\n",
+	 FLAG(command, PCI_PCIX_COMMAND_DPERE),
+	 FLAG(command, PCI_PCIX_COMMAND_ERO),
+	 ((command & PCI_PCIX_COMMAND_MAX_MEM_READ_BYTE_COUNT) >> 2U),
+	 ((command & PCI_PCIX_COMMAND_MAX_OUTSTANDING_SPLIT_TRANS) >> 4U));
+  printf("\t\tStatus: Bus=%u Dev=%u Func=%u 64bit%c 133MHz%c SCD%c USC%c, DC=%s, DMMRBC=%u, DMOST=%u, DMCRS=%u, RSCEM%c",
+	 ((status >> 8) & 0xffU), // bus
+	 ((status >> 3) & 0x1fU), // dev
+	 (status & PCI_PCIX_BRIDGE_STATUS_FUNCTION), // function
+	 FLAG(status, PCI_PCIX_STATUS_64BIT),
+	 FLAG(status, PCI_PCIX_STATUS_133MHZ),
+	 FLAG(status, PCI_PCIX_STATUS_SC_DISCARDED),
+	 FLAG(status, PCI_PCIX_STATUS_UNEXPECTED_SC),
+	 ((status & PCI_PCIX_STATUS_DEVICE_COMPLEXITY) ? "bridge" : "simple"),
+	 ((status >> 21) & 3U),
+	 ((status >> 23) & 7U),
+	 ((status >> 26) & 7U),
+	 FLAG(status, PCI_PCIX_STATUS_RCVD_SC_ERR_MESS));
+}
+
+static void
+show_pcix_bridge(struct device *d, int where)
+{
+  u16 secstatus;
+  u32 status, upstcr, downstcr;
+  printf("PCI-X bridge device.\n");
+  if (verbose < 2)
+    return;
+  secstatus = get_conf_word(d, where + PCI_PCIX_BRIDGE_SEC_STATUS);
+  printf("\t\tSecondary Status: 64bit%c, 133MHz%c, SCD%c, USC%c, SCO%c, SRD%c Freq=%d\n",
+	 FLAG(secstatus, PCI_PCIX_BRIDGE_SEC_STATUS_64BIT),
+	 FLAG(secstatus, PCI_PCIX_BRIDGE_SEC_STATUS_133MHZ),
+	 FLAG(secstatus, PCI_PCIX_BRIDGE_SEC_STATUS_SC_DISCARDED),
+	 FLAG(secstatus, PCI_PCIX_BRIDGE_SEC_STATUS_UNEXPECTED_SC),
+	 FLAG(secstatus, PCI_PCIX_BRIDGE_SEC_STATUS_SC_OVERRUN),
+	 FLAG(secstatus, PCI_PCIX_BRIDGE_SEC_STATUS_SPLIT_REQUEST_DELAYED),
+	 ((secstatus >> 6) & 7));
+  status = get_conf_long(d, where + PCI_PCIX_BRIDGE_STATUS);
+  printf("\t\tStatus: Bus=%u Dev=%u Func=%u 64bit%c 133MHz%c SCD%c USC%c, SCO%c, SRD%c\n", 
+	 ((status >> 8) & 0xff), // bus
+	 ((status >> 3) & 0x1f), // dev
+	 (status & PCI_PCIX_BRIDGE_STATUS_FUNCTION), // function
+	 FLAG(status, PCI_PCIX_BRIDGE_STATUS_64BIT),
+	 FLAG(status, PCI_PCIX_BRIDGE_STATUS_133MHZ),
+	 FLAG(status, PCI_PCIX_BRIDGE_STATUS_SC_DISCARDED),
+	 FLAG(status, PCI_PCIX_BRIDGE_STATUS_UNEXPECTED_SC),
+	 FLAG(status, PCI_PCIX_BRIDGE_STATUS_SC_OVERRUN),
+	 FLAG(status, PCI_PCIX_BRIDGE_STATUS_SPLIT_REQUEST_DELAYED));
+  upstcr = get_conf_long(d, where + PCI_PCIX_BRIDGE_UPSTREAM_SPLIT_TRANS_CTRL);
+  printf("\t\t: Upstream: Capacity=%u, Commitment Limit=%u\n",
+	 (upstcr & PCI_PCIX_BRIDGE_STR_CAPACITY),
+	 (upstcr >> 16) & 0xffff);
+  downstcr = get_conf_long(d, where + PCI_PCIX_BRIDGE_DOWNSTREAM_SPLIT_TRANS_CTRL);
+  printf("\t\t: Downstream: Capacity=%u, Commitment Limit=%u\n",
+	 (downstcr & PCI_PCIX_BRIDGE_STR_CAPACITY),
+	 (downstcr >> 16) & 0xffff);
+}
+
+static void
+show_pcix(struct device *d, int where)
+{
+  switch (d->dev->hdrtype)
+    {
+    case PCI_HEADER_TYPE_NORMAL:
+      show_pcix_nobridge(d, where);
+      break;
+    case PCI_HEADER_TYPE_BRIDGE:
+      show_pcix_bridge(d, where);
+      break;
+    }
+}
+
 static void
 show_rom(struct device *d)
 {
@@ -549,6 +639,9 @@ show_caps(struct device *d)
 	    case PCI_CAP_ID_MSI:
 	      show_msi(d, where, cap);
 	      break;
+	    case PCI_CAP_ID_PCIX:
+	      show_pcix(d, where);
+	      break;
 	    default:
 	      printf("#%02x [%04x]\n", id, cap);
 	    }
@@ -733,11 +826,7 @@ show_verbose(struct device *d)
     {
     case PCI_HEADER_TYPE_NORMAL:
       if (class == PCI_CLASS_BRIDGE_PCI)
-	{
-	badhdr:
-	  printf("\t!!! Header type %02x doesn't match class code %04x\n", htype, class);
-	  return;
-	}
+	printf("\t!!! Invalid class %04x for header type %02x\n", class, htype);
       max_lat = get_conf_byte(d, PCI_MAX_LAT);
       min_gnt = get_conf_byte(d, PCI_MIN_GNT);
       subsys_v = get_conf_word(d, PCI_SUBSYSTEM_VENDOR_ID);
@@ -745,13 +834,13 @@ show_verbose(struct device *d)
       break;
     case PCI_HEADER_TYPE_BRIDGE:
       if (class != PCI_CLASS_BRIDGE_PCI)
-	goto badhdr;
+	printf("\t!!! Invalid class %04x for header type %02x\n", class, htype);
       irq = int_pin = min_gnt = max_lat = 0;
       subsys_v = subsys_d = 0;
       break;
     case PCI_HEADER_TYPE_CARDBUS:
       if ((class >> 8) != PCI_BASE_CLASS_BRIDGE)
-	goto badhdr;
+	printf("\t!!! Invalid class %04x for header type %02x\n", class, htype);
       min_gnt = max_lat = 0;
       subsys_v = get_conf_word(d, PCI_CB_SUBSYSTEM_VENDOR_ID);
       subsys_d = get_conf_word(d, PCI_CB_SUBSYSTEM_ID);
@@ -796,12 +885,18 @@ show_verbose(struct device *d)
 	     FLAG(status, PCI_STATUS_DETECTED_PARITY));
       if (cmd & PCI_COMMAND_MASTER)
 	{
-	  printf("\tLatency: ");
-	  if (min_gnt)
-	    printf("%d min, ", min_gnt);
-	  if (max_lat)
-	    printf("%d max, ", max_lat);
-	  printf("%d set", latency);
+	  printf("\tLatency: %d", latency);
+	  if (min_gnt || max_lat)
+	    {
+	      printf(" (");
+	      if (min_gnt)
+		printf("%dns min", min_gnt*250);
+	      if (min_gnt && max_lat)
+		printf(", ");
+	      if (max_lat)
+		printf("%dns max", max_lat*250);
+	      putchar(')');
+	    }
 	  if (cache_line)
 	    printf(", cache line size %02x", cache_line);
 	  putchar('\n');
